@@ -1,8 +1,9 @@
 from .utils import (hierarchical_path, columns_with_distinct_values,
                     create_pairs,
-                    column_use,
                     figures_to_html)
-from datamallet.tabular.utils import (check_dataframe,check_numeric, check_columns,
+from datamallet.tabular.utils import (check_dataframe,check_categorical,
+                                      check_numeric,
+                                      check_columns,
                                       extract_numeric_cols,
                                       calculate_correlation)
 import pandas as pd
@@ -22,24 +23,26 @@ def create_pie(df,
     :param list_of_categorical_columns:list of column names which have categorical data.
             Output of function columns_with_distinct_values(df=self.df, categorical_only=True,
                                                         maximum_number_distinct_values=maximum_number_sectors)
+            Also output of extract_categorical_cols in tabular/utils module
     :param create_html:boolean, whether the figures should be converted to HTML or not
     :param hole:boolean, hole in the pie chart
     :param filename:str, a suitable name for the produced html file, exclude the extension
     :return: list which contains graph objects
     """
+    assert isinstance(df, pd.DataFrame), "df must be a pandas dataframe"
     assert isinstance(numeric_cols, list), "numeric_cols must be a list"
     assert isinstance(list_of_categorical_columns, list), "list_of_categorical_columns must be a list"
-    assert isinstance(create_html, bool), "create_html must be a boolean"
-    assert isinstance(hole, bool), "hole must be a boolean"
-    assert isinstance(filename, str), "filename must be a string with a dot or an extension"
     assert len(list_of_categorical_columns) != 0, "list_of_categorical_columns must not be empty"
     assert len(numeric_cols) != 0, "numeric_cols must not be empty"
-    assert isinstance(df, pd.DataFrame), "df must be a pandas dataframe"
+    assert isinstance(create_html, bool), "create_html must be a boolean"
+    assert isinstance(hole, bool), "hole must be a boolean"
+    assert isinstance(filename, str), "filename must be a string without a dot or an extension"
     assert '.' not in filename, "filename doesn't need an extension"
 
     figure_list = list()
 
-    if check_numeric(df=df,column_list=numeric_cols) and check_columns(df=df,column_list=numeric_cols):
+    if check_numeric(df=df,column_list=numeric_cols) and check_categorical(df=df,
+                                                                           column_list=list_of_categorical_columns):
 
         for value in numeric_cols:
             for name in list_of_categorical_columns:
@@ -334,12 +337,11 @@ def create_correlation_plot(df,
     assert isinstance(correlation_method, str), "correlation method must be a string"
     assert isinstance(filename, str), "filename must be a string with a dot or an extension"
     assert '.' not in filename, "filename doesn't need an extension"
+    assert correlation_method in ['pearson', 'kendall', 'spearman']
 
     figure_list = list()
 
-    available_method = correlation_method in ['pearson', 'kendall', 'spearman']
-
-    if available_method and len(extract_numeric_cols(df=df))>1:
+    if len(extract_numeric_cols(df=df))>1:
 
         correlation = calculate_correlation(df=df, method=correlation_method)
 
@@ -381,17 +383,16 @@ def create_histogram(df,
     """
     assert isinstance(df, pd.DataFrame), "df must be a pandas dataframe"
     assert isinstance(numeric_cols, list), "numeric_cols must be a list"
-    assert isinstance(filename, str), "filename must be a string with a dot or an extension"
-    assert isinstance(create_html, bool), "create_html must be a boolean"
     assert isinstance(nbins, int), "nbins must be an integer"
+    assert marginal in ['rug','box','violin','histogram',None], "marginal options 'rug','box','violin','histogram'"
     assert isinstance(cumulative, bool), "cumulative must be a boolean"
-    assert marginal in ['rug','box','violin','histogram',None], "marginal must be one of 'rug','box','violin','histogram'"
+    assert histfunc in ['count', 'sum', 'avg', 'min',
+                        'max'], "histfunc must be one of 'count', 'sum', 'avg','min','max' "
+    assert histnorm in ['percent', 'probability', 'density', 'probability density', None]
+    assert isinstance(filename, str), "filename must be a string with a dot or an extension"
     assert '.' not in filename, "filename doesn't need an extension"
-    assert isinstance(histfunc, str),"histfunc must be a string"
-    assert isinstance(orientation,str)
+    assert isinstance(create_html, bool), "create_html must be a boolean"
     assert orientation in ['v','h']
-    assert histnorm in ['percent','probability','density','probability density',None]
-    assert histfunc in ['count', 'sum', 'avg','min','max'], "histfunc must be one of 'count', 'sum', 'avg','min','max' "
 
     figure_list = list()
 
@@ -417,79 +418,83 @@ def create_histogram(df,
 
 def create_scatter(df,
                    col_types,
-                   basic=True,
                    filename='scatter',
                    marginal_x=None,
                    marginal_y=None,
                    log_x=False,
                    log_y=False,
+                   orientation='v',
+                   opacity=1.0,
+                   maximum_color_groups=5,
                    create_html=True):
     """
-
     :param df:
     :param col_types: dictionary that contains mapping of column type to list of column names
                     It is the output of extract_col_types in tabular module
-    :param basic:boolean, whether to create a basic scatterplot or not
     :param filename:
     :param marginal_x:options for including marginal charts on x axis
     :param marginal_y:options for including marginal charts on y axis
     :param log_x:boolean, whether to create a log axis
     :param log_y:boolean, whether to create a log axis
+    :param orientation: str, the orientation of the figure, 'v' or 'h'
     :param create_html: boolean, whether to create an html file or not
+    :param opacity: float, value between 0 and 1. Sets the opacity for markers
+    :param maximum_color_groups: int, maximum number of color groups in the scatter plot
     :return:
     """
     assert isinstance(df, pd.DataFrame), "df must be a pandas dataframe"
     assert isinstance(col_types, dict), "col_types must be a dictionary with column " \
                                         "name as keys and column type as value"
-    assert isinstance(create_html, bool), "create_html must be a boolean"
+    assert 'numeric' in col_types.keys(), "col_types dictionary missing key numeric"
+    assert 'object' in col_types.keys(), "col_types dictionary missing key object"
+    assert 'boolean' in col_types.keys(), "col_types dictionary missing key boolean"
+    assert 'categorical' in col_types.keys(), "col_types dictionary missing key categorical"
+    assert 'datetime' in col_types.keys(), "col_types dictionary missing key datetime"
+    assert 'timedelta' in col_types.keys(), "col_types dictionary missing key timedelta"
     assert isinstance(filename, str), "filename must be a string with a dot or an extension"
     assert '.' not in filename, "filename doesn't need an extension"
-    numeric_cols = col_types['numeric']
-    plot_pairs = create_pairs(df, numeric_cols=numeric_cols)
+    assert isinstance(log_y, bool)
+    assert isinstance(log_x, bool)
+    assert marginal_x in ['rug', 'box', 'violin', 'histogram',None], "marginal must be one of 'rug'," \
+                                                                     "'box','violin','histogram'"
+    assert marginal_y in ['rug', 'box', 'violin', 'histogram',None], "marginal must be one of 'rug'," \
+                                                                     "'box','violin','histogram'"
+    assert isinstance(opacity, float), "opacity must be a float"
+    assert opacity <= 1.0, "opacity must be between 0 and 1"
+    assert isinstance(create_html, bool), "create_html must be a boolean"
+    assert orientation in ['v', 'h']
+    assert isinstance(maximum_color_groups, int)
 
     figure_list = list()
+    numeric_cols = col_types['numeric']
 
-    if basic:
-        for pair in plot_pairs:
-            x,y = pair
+    if len(numeric_cols) < 2:
+        # scatter plots need at least 2 columns of numeric type
+        return figure_list
+    plot_pairs = create_pairs(df, numeric_cols=numeric_cols)
+
+    columns_with_distinct = columns_with_distinct_values(df,
+                                                         maximum_number_distinct_values=maximum_color_groups,
+                                                         categorical_only=True)
+    if len(columns_with_distinct) == 0:
+        columns_with_distinct.append(None)
+
+    for pair in plot_pairs:
+        x,y = pair
+        for color in columns_with_distinct:
             plot = px.scatter(data_frame=df,
                               x=x,
                               y=y,
                               log_y=log_y,
                               log_x=log_x,
+                              opacity=opacity,
+                              orientation=orientation,
                               marginal_x=marginal_x,
                               marginal_y=marginal_y,
-                              title='Plot of {} vs {}'.format(x,y))
+                              color=color,
+                              title='Plot of {} vs {} with color {}'.format(x,y,color))
             figure_list.append(plot)
 
-    else:
-        cols_use_dict = column_use(df, col_types=col_types, threshold=5)
-        name_list = cols_use_dict['name']
-        hue_list = cols_use_dict['hue']
-
-        if len(name_list) == 0:
-            name_list.append(None)
-
-        if len(hue_list) == 0:
-            hue_list.append(None)
-
-        for pair in plot_pairs:
-            x,y = pair
-            for names in name_list:
-                for hue in hue_list:
-                    plot = px.scatter(data_frame=df,
-                                      x=x,
-                                      y=y,
-                                      color=hue,
-                                      log_y=log_y,
-                                      log_x=log_x,
-                                      marginal_x=marginal_x,
-                                      marginal_y=marginal_y,
-                                      hover_name=names,
-                                      title='Plot of {} vs {} with color {} and hover name {}'.format(x, y, hue,names))
-
-                    figure_list.append(plot)
-
     if create_html:
         figures_to_html(figs=figure_list, filename=filename)
 
